<?php
/*
 * This file is part of EC-CUBE
 *
 * Copyright(c) 2000-2013 LOCKON CO.,LTD. All Rights Reserved.
 *
 * http://www.lockon.co.jp/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

// {{{ requires
require_once CLASS_REALDIR . 'pages/admin/order/LC_Page_Admin_Order.php';
// ASJペイメントモジュール
require_once(MODULE_REALDIR . 'mdl_asjpayment/include.php');

/**
 * 受注管理 のページクラス(拡張).
 *
 * LC_Page_Admin_Order をカスタマイズする場合はこのクラスを編集する.
 *
 * @package Page
 * @author LOCKON CO.,LTD.
 * @version $Id: LC_Page_Admin_Order_Ex.php,v 1.5 2014/11/18 07:33:58 iwashi Exp $
 */
class LC_Page_Admin_Order_Ex extends LC_Page_Admin_Order {
	/**
	 * Page を初期化する.
	 */
	function init() {
		parent::init();
	}

	/**
	 * Page のプロセス.
	 */
	function process() {
		parent::process();
	}

	/**
	 * Page のアクション.
	 */
	function action() {
		$objAsjDate = new SC_Date_Ex();
		$objAsjDate->setStartYear(DATE('Y') - 1);
		$objAsjDate->setEndYear(DATE('Y'));
		$this->arrAsjRegistYear = $objAsjDate->getYear();

		$objFormParam = new SC_FormParam_Ex();
		$this->lfInitParam($objFormParam);
		$objFormParam->setParam($_POST);
		$this->arrHidden = $objFormParam->getSearchArray();
		$this->arrForm = $objFormParam->getFormParamList();

		$objPurchase = new SC_Helper_Purchase_Ex();
		$order = 'dtb_order.update_date DESC';
		// 削除
		if($this->getMode() == 'delete') {
			$order_id = $objFormParam->getValue('order_id');
			$objPurchase->cancelOrder($order_id, ORDER_CANCEL, true);
		// 入金日登録
		}
		switch($this->getMode()) {
		case 'delete':
		// 検索パラメーター生成後に処理実行するため breakしない
		case 'csv':
		case 'asjcsv':
		case 'asjpaymentdate':
		case 'delete_all':
		// 検索パラメーターの生成
		case 'search':
			$objFormParam->convParam();
			$objFormParam->trimParam();
			$this->arrErr = $this->lfCheckError($objFormParam);
			$arrParam = $objFormParam->getHashArray();
			if(count($this->arrErr) == 0) {
				$where = 'dtb_order.del_flg = 0';
				$arrWhereVal = array();
				foreach($arrParam as $key => $val) {
					if($val == '') {
						continue;
					}
//					$this->buildQuery($key, $where, $arrWhereVal, $objFormParam);
					$this->myBuildQuery($key, $where, $arrWhereVal, $objFormParam);
				}
				// 処理を実行
				switch($this->getMode()) {
				// CSVを送信する。
				case 'csv':
					$this->doOutputCSV($where, $arrWhereVal, $order);
					SC_Response_Ex::actionExit();
					break;
				case 'asjcsv':
					$this->arrErr = $this->lfCheckErrorAsjCsv($objFormParam);
					if(count($this->arrErr) == 0) {
						$this->doOutputAsjCSV($order, $objFormParam);
						SC_Response_Ex::actionExit();
					}
					break;
				// 全件削除(ADMIN_MODE)
				case 'delete_all':
					$page_max = 0;
					$arrResults = $this->findOrders($where, $arrWhereVal, $page_max, 0, $order);
					foreach($arrResults as $element) {
						$objPurchase->cancelOrder($element['order_id'], ORDER_CANCEL, true);
					}
					break;
				//入金日登録
				case 'asjpaymentdate':
					$this->arrErr = $this->lfCheckErrorAsjCsv($objFormParam);
					if(count($this->arrErr) == 0) {
						$this->updatePaymentDate($order, $objFormParam);
					}
				// 検索実行
				default:
					// 行数の取得
					$this->tpl_linemax = $this->getNumberOfLines($where, $arrWhereVal);
					// ページ送りの処理
					$page_max = SC_Utils_Ex::sfGetSearchPageMax($objFormParam->getValue('search_page_max'));
					// ページ送りの取得
					$objNavi = new SC_PageNavi_Ex($this->arrHidden['search_pageno']
												, $this->tpl_linemax, $page_max
												, 'fnNaviSearchPage', NAVI_PMAX);
					$this->arrPagenavi = $objNavi->arrPagenavi;
					// 検索結果の取得
					$this->arrResults = $this->findOrders($where, $arrWhereVal, $page_max, $objNavi->start_row, $order);
					break;
				}
			}
			break;
		default:
			break;
		}

	}

	/**
	 * デストラクタ.
	 */
	function destroy() {
		parent::destroy();
	}

	/**
	 * パラメーター情報の初期化
	 */
	function lfInitParam(&$objFormParam) {
		parent::lfInitParam($objFormParam);

		switch($this->getMode()) {
		case 'asjcsv':
			$objFormParam->addParam('決済確定日 年', 'asjcsv_year', INT_LEN, 'n', array('MAX_LENGTH_CHECK', 'NUM_CHECK'));
			$objFormParam->addParam('決済確定日 月', 'asjcsv_month', INT_LEN, 'n', array('MAX_LENGTH_CHECK', 'NUM_CHECK'));
			$objFormParam->addParam('決済確定日 日', 'asjcsv_day', INT_LEN, 'n', array('MAX_LENGTH_CHECK', 'NUM_CHECK'));
			$objFormParam->addParam('確定日', 'complete_day_order_id', INT_LEN, 'n', array('MAX_LENGTH_CHECK', 'NUM_CHECK'));
			break;
		case 'asjpaymentdate':
			$objFormParam->addParam('入金日 年', 'asjcsv_year', INT_LEN, 'n', array('MAX_LENGTH_CHECK', 'NUM_CHECK'));
			$objFormParam->addParam('入金日 月', 'asjcsv_month', INT_LEN, 'n', array('MAX_LENGTH_CHECK', 'NUM_CHECK'));
			$objFormParam->addParam('入金日 日', 'asjcsv_day', INT_LEN, 'n', array('MAX_LENGTH_CHECK', 'NUM_CHECK'));
			$objFormParam->addParam('入金日', 'complete_day_order_id', INT_LEN, 'n', array('MAX_LENGTH_CHECK', 'NUM_CHECK'));
			break;
		}
	}

	/**
	 * 入力内容のチェックを行う.
	 *
	 * @param SC_FormParam $objFormParam SC_FormParam インスタンス
	 * @return void
	 */
	function lfCheckErrorAsjCsv(&$objFormParam) {
		$objErr = new SC_CheckError_Ex($objFormParam->getHashArray());
		// 決済確定日
		$objErr->doFunc(array('決済確定日', 'asjcsv_year', 'asjcsv_month', 'asjcsv_day')
						, array('EXIST_CHECK', 'CHECK_DATE'));
		$arrVal = $objFormParam->getValue('complete_day_order_id');
		$str = '';
		foreach($arrVal as $key => $val) {
			if(strlen($val) <= 0 || !is_numeric($val)) {
				array_push($objErr->arrErr, "complete_day_order_id", "確定日の値が不正です。");
			}
		}
		return $objErr->arrErr;
	}

	/**
	 * CSV データを構築して取得する.
	 * order指定が必須のため、他の検索条件を使用しません。
	 */
	function doOutputAsjCSV($order, &$objFormParam) {
		$sql = 'select dtb_order.order_id';
		$sql .= sprintf(", '%04d/%02d/%02d'"
							, $objFormParam->getValue('asjcsv_year')
							, $objFormParam->getValue('asjcsv_month')
							, $objFormParam->getValue('asjcsv_day'));
		$sql .= ' from dtb_order';
		$sql .= ', dtb_payment';
		$sql .= ' where dtb_order.payment_id = dtb_payment.payment_id';
		$sql .= " and dtb_payment.module_code = '" . MDL_ASJPAYMENT_CODE . "'";
		$arrVal = $objFormParam->getValue('complete_day_order_id');
		$str = '';
		foreach($arrVal as $key => $val) {
			if(strlen($str) > 0) {
				$str .= ',' . $val;
			} else {
				$str = $val;
			}
		}
		if(strlen($str) > 0) {
			$sql .= ' and dtb_order.order_id in(' . $str . ')';
		}
		if(strlen($order) > 0) {
			$sql .= ' order by ' . $order;
		}
		$arrVal = array();
		$objCSV = new SC_Helper_CSV_Ex();
		$objCSV->sfDownloadCsvFromSql($sql, $arrVal, 'order', array('注文番号', '決済確定日'), true);

		//決済確定日 現在時間 で登録
		$commit_date = sprintf("%04d/%02d/%02d %s"
											, $objFormParam->getValue('asjcsv_year')
											, $objFormParam->getValue('asjcsv_month')
											, $objFormParam->getValue('asjcsv_day'), date("H:i:s"));
		$objQuery =& SC_Query_Ex::getSingletonInstance();
		//CSVで出力したorder_idを発送済に変更
		$arrVal = $objQuery->getAll($sql);
		$sql =  'update dtb_order';
		$sql .= " set commit_date = '" . $commit_date . "'";
		$sql .= ', status = ' . ORDER_DELIV;
		$sql .= ' where status != ' . ORDER_DELIV;
		$sql .= ' and status != ' . ORDER_CANCEL;
		$str = '';
		foreach($arrVal as $key => $val) {
			if(strlen($str) > 0) {
				$str .= ',' . $val['order_id'];
			} else {
				$str = $val['order_id'];
			}
		}
		if(strlen($str) > 0) {
			$sql .= ' and order_id in(' . $str . ')';
			$objQuery->query($sql);
		}
	}

	/**
	 * 入金日を登録する
	 * order指定が必須のため、他の検索条件を使用しません。
	 */
	function updatePaymentDate($order, &$objFormParam) {
		$sql = 'select dtb_order.order_id';
		$sql .= ', dtb_order.create_date';
		$sql .= ' from dtb_order';
		$sql .= ', dtb_payment';
		$sql .= ' where dtb_order.payment_id = dtb_payment.payment_id';
		$sql .= " and dtb_payment.module_code = '" . MDL_ASJPAYMENT_CODE . "'";
		$arrVal = $objFormParam->getValue('complete_day_order_id');
		$str = '';
		foreach($arrVal as $key => $val) {
			if(strlen($str) > 0) {
				$str .= ',' . $val;
			} else {
				$str = $val;
			}
		}
		if(strlen($str) > 0) {
			$sql .= ' and dtb_order.order_id in(' . $str . ')';
		}
		if(strlen($order) > 0) {
			$sql .= ' order by ' . $order;
		}
		$arrVal = array();
		//入金日
		$payment_date = sprintf("%04d/%02d/%02d 00:00:00"
											, $objFormParam->getValue('asjcsv_year')
											, $objFormParam->getValue('asjcsv_month')
											, $objFormParam->getValue('asjcsv_day'));
		//現在日
		$nowDate = new DateTime(date('Y-m-d 00:00:00', time()));
		$objQuery =& SC_Query_Ex::getSingletonInstance();
		//CSVで出力したorder_idを発送済に変更
		$arrVal = $objQuery->getAll($sql);
		$sql =  'update dtb_order';
		$sql .= " set payment_date = '" . $payment_date . "'";
		$sql .= ', status = ' . ORDER_PRE_END;
		$sql .= ' where status != ' . ORDER_DELIV;
		$sql .= ' and status != ' . ORDER_CANCEL;
		$str = '';
		foreach($arrVal as $key => $val) {
			if(!is_null($val['create_date']) && !empty($val['create_date'])) {
				$createDate = new DateTime($val['create_date']);
				if($createDate->format('Ymd') <= $nowDate->format('Ymd')) {
					if(strlen($str) > 0) {
						$str .= ',' . $val['order_id'];
					} else {
						$str = $val['order_id'];
					}
				}
			}
		}
		if(strlen($str) > 0) {
			$sql .= ' and order_id in(' . $str . ')';
			$objQuery->query($sql);
		}
	}

	/**
	 * buildQueryをfindOrdersで通るように変更
	 */
	public function myBuildQuery($key, &$where, &$arrValues, &$objFormParam, &$objDb) {
		$dbFactory = SC_DB_DBFactory_Ex::getInstance();
		switch ($key) {
		case 'search_product_name':
			$where .= ' AND EXISTS (SELECT 1 FROM dtb_order_detail od WHERE od.order_id = dtb_order.order_id AND od.product_name LIKE ?)';
			$arrValues[] = sprintf('%%%s%%', $objFormParam->getValue($key));
			break;
		case 'search_order_name':
			$where .= ' AND ' . $dbFactory->concatColumn(array('order_name01', 'order_name02')) . ' LIKE ?';
			$arrValues[] = sprintf('%%%s%%', preg_replace('/[ 　]/u', '', $objFormParam->getValue($key)));
			break;
		case 'search_order_kana':
			$where .= ' AND ' . $dbFactory->concatColumn(array('order_kana01', 'order_kana02')) . ' LIKE ?';
			$arrValues[] = sprintf('%%%s%%', preg_replace('/[ 　]/u', '', $objFormParam->getValue($key)));
			break;
		case 'search_order_id1':
			$where .= ' AND dtb_order.order_id >= ?';
			$arrValues[] = sprintf('%d', $objFormParam->getValue($key));
			break;
		case 'search_order_id2':
			$where .= ' AND dtb_order.order_id <= ?';
			$arrValues[] = sprintf('%d', $objFormParam->getValue($key));
			break;
		case 'search_order_sex':
			$tmp_where = '';
			foreach ($objFormParam->getValue($key) as $element) {
				if ($element != '') {
					if (SC_Utils_Ex::isBlank($tmp_where)) {
						$tmp_where .= ' AND (dtb_order.order_sex = ?';
					} else {
						$tmp_where .= ' OR dtb_order.order_sex = ?';
					}
					$arrValues[] = $element;
				}
			}

			if (!SC_Utils_Ex::isBlank($tmp_where)) {
				$tmp_where .= ')';
				$where .= " $tmp_where ";
			}
			break;
		case 'search_order_tel':
			$where .= ' AND (' . $dbFactory->concatColumn(array('order_tel01', 'order_tel02', 'order_tel03')) . ' LIKE ?)';
			$arrValues[] = sprintf('%%%d%%', preg_replace('/[()-]+/','', $objFormParam->getValue($key)));
			break;
		case 'search_order_email':
			$where .= ' AND dtb_order.order_email LIKE ?';
			$arrValues[] = sprintf('%%%s%%', $objFormParam->getValue($key));
			break;
		case 'search_payment_id':
			$tmp_where = '';
			foreach ($objFormParam->getValue($key) as $element) {
				if ($element != '') {
					if ($tmp_where == '') {
						$tmp_where .= ' AND (dtb_order.payment_id = ?';
					} else {
						$tmp_where .= ' OR dtb_order.payment_id = ?';
					}
					$arrValues[] = $element;
				}
			}

			if (!SC_Utils_Ex::isBlank($tmp_where)) {
				$tmp_where .= ')';
				$where .= " $tmp_where ";
			}
			break;
		case 'search_total1':
			$where .= ' AND dtb_order.total >= ?';
			$arrValues[] = sprintf('%d', $objFormParam->getValue($key));
			break;
		case 'search_total2':
			$where .= ' AND dtb_order.total <= ?';
			$arrValues[] = sprintf('%d', $objFormParam->getValue($key));
			break;
		case 'search_sorderyear':
			$date = SC_Utils_Ex::sfGetTimestamp($objFormParam->getValue('search_sorderyear'),
												$objFormParam->getValue('search_sordermonth'),
												$objFormParam->getValue('search_sorderday'));
			$where.= ' AND dtb_order.create_date >= ?';
			$arrValues[] = $date;
			break;
		case 'search_eorderyear':
			$date = SC_Utils_Ex::sfGetTimestamp($objFormParam->getValue('search_eorderyear'),
												$objFormParam->getValue('search_eordermonth'),
												$objFormParam->getValue('search_eorderday'), true);
			$where.= ' AND dtb_order.create_date <= ?';
			$arrValues[] = $date;
			break;
		case 'search_supdateyear':
			$date = SC_Utils_Ex::sfGetTimestamp($objFormParam->getValue('search_supdateyear'),
												$objFormParam->getValue('search_supdatemonth'),
												$objFormParam->getValue('search_supdateday'));
			$where.= ' AND dtb_order.update_date >= ?';
			$arrValues[] = $date;
			break;
		case 'search_eupdateyear':
			$date = SC_Utils_Ex::sfGetTimestamp($objFormParam->getValue('search_eupdateyear'),
												$objFormParam->getValue('search_eupdatemonth'),
												$objFormParam->getValue('search_eupdateday'), true);
			$where.= ' AND dtb_order.update_date <= ?';
			$arrValues[] = $date;
			break;
		case 'search_sbirthyear':
			$date = SC_Utils_Ex::sfGetTimestamp($objFormParam->getValue('search_sbirthyear'),
												$objFormParam->getValue('search_sbirthmonth'),
												$objFormParam->getValue('search_sbirthday'));
			$where.= ' AND dtb_order.order_birth >= ?';
			$arrValues[] = $date;
			break;
		case 'search_ebirthyear':
			$date = SC_Utils_Ex::sfGetTimestamp($objFormParam->getValue('search_ebirthyear'),
												$objFormParam->getValue('search_ebirthmonth'),
												$objFormParam->getValue('search_ebirthday'), true);
			$where.= ' AND dtb_order.order_birth <= ?';
			$arrValues[] = $date;
			break;
		case 'search_order_status':
			$where.= ' AND dtb_order.status = ?';
			$arrValues[] = $objFormParam->getValue($key);
			break;
		default:
			break;
		}
	}

	/**
	 * 受注を検索する.
	 *
	 * @param string $where 検索条件の WHERE 句
	 * @param array $arrValues 検索条件のパラメーター
	 * @param integer $limit 表示件数
	 * @param integer $offset 開始件数
	 * @param string $order 検索結果の並び順
	 * @return array 受注の検索結果
	 */
	function findOrders($where, $arrValues, $limit, $offset, $order) {
		// 登録・更新日検索用
		$objQuery =& SC_Query_Ex::getSingletonInstance();
		$objQuery->setLimitOffset($limit, $offset);
		$objQuery->setOrder($order);
		$col = 'dtb_order.*';
		$col .= ', case when dtb_order.payment_id = dtb_payment.payment_id then 1';
		$col .= ' else 0';
		$col .= ' end as bComplete';
		$table = 'dtb_order left join dtb_payment';
		$table .= ' on dtb_order.payment_id = dtb_payment.payment_id';
		$table .= " and dtb_payment.module_code = '" . MDL_ASJPAYMENT_CODE . "'";
		$ret = $objQuery->select($col, $table, $where, $arrValues);
		return $ret;
	}
}
